/* Thanks for the icon, GNOME! They make totally kick-ass icons. First time I get to use one, though. For more cool stuff, see http://www.gnome.org */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.Win32;

namespace TimezoneEdit
{
	public partial class Form1 : Form
	{
		[DllImport("coredll.dll", EntryPoint = "SetTimeZoneInformation", SetLastError=true)]
		public static extern UInt32 SetTimeZoneInformation(ref TIME_ZONE_INFORMATION TZI);

		[DllImport("coredll.dll", EntryPoint = "GetTimeZoneInformation")]
		public static extern UInt32 GetTimeZoneInformation(out TIME_ZONE_INFORMATION TZI);

		public const uint FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;

		[DllImport("coredll.dll")]
		public static extern uint FormatMessage(
		 uint dwFlags, string lpSource, uint dwMessageId,
		 uint dwLanguageId, StringBuilder lpBuffer, uint nSize,
		 string[] Arguments);


		[StructLayout(LayoutKind.Sequential, Size=16)]
		public struct SYSTEMTIME {
			public ushort wYear;
			public ushort wMonth;
			public ushort wDayOfWeek;
			public ushort wDay;
			public ushort wHour;
			public ushort wMinute;
			public ushort wSecond;
			public ushort wMilliseconds;
		}

		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Unicode, Size = 108)]
		public struct TIME_ZONE_INFORMATION
		{
			[MarshalAs(UnmanagedType.I4)]
			public int Bias;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
			public string StandardName;
			[MarshalAs(UnmanagedType.Struct, SizeConst = 16)]
			public SYSTEMTIME StandardDate;
			[MarshalAs(UnmanagedType.I4)]
			public int StandardBias;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
			public string DaylightName;
			[MarshalAs(UnmanagedType.Struct, SizeConst = 16)]
			public SYSTEMTIME DaylightDate;
			[MarshalAs(UnmanagedType.I4)]
			public int DaylightBias;
		}

		private string SystemTimeToString(SYSTEMTIME systime)
		{
			return String.Format(
				"{0}-{1}-{2} DOW:{3} {4}:{5}:{6}.{7}", 
				systime.wYear, systime.wMonth, systime.wDay, 
				systime.wDayOfWeek, 
				systime.wHour, systime.wMinute, systime.wSecond, systime.wMilliseconds);
		}

		private string TZIToString(TIME_ZONE_INFORMATION tzi) {
			return String.Format(
				"StdName: {0}{1}StdBias: {2}{1}StdDate: {3}{1}DSTName: {6}{1}DSTBias: {4}{1}DSTDate: {5}{1}{1}Bias: {7}{1}",
				tzi.StandardName, "\r\n", tzi.StandardBias, SystemTimeToString(tzi.StandardDate),
				tzi.DaylightBias, SystemTimeToString(tzi.DaylightDate), tzi.DaylightName, tzi.Bias);
		}

		public Form1()
		{
			InitializeComponent();
		}

		private TIME_ZONE_INFORMATION tzi;

		private void saveMenuItem_Click(object sender, EventArgs e)
		{
			DialogResult result = MessageBox.Show("Update Israel DST?", "Confirm Change DST", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2);
			if (result == DialogResult.Yes)
			{
				this.logTextBox.Text += "\r\nSetting...";

                tzi.DaylightDate.wYear = 0;      // always 0. See http://msdn.microsoft.com/en-us/library/aa908812.aspx
				tzi.DaylightDate.wDayOfWeek = (ushort)this.wdDSTComboBox.SelectedIndex;
                                                 // 0=Sun, ... 5=Friday
                tzi.DaylightDate.wDay = (ushort)this.posDSTComboBox.SelectedIndex;	     
                                                 // 0=first, 1=second, .. 4=fourth, 5=last Friday in the month
                tzi.DaylightDate.wMonth = (ushort)(this.monthDSTComboBox.SelectedIndex+1);	 
                                                 // 1=Jan, ... 3=March .. 12=Dec
				tzi.DaylightDate.wHour = 2;		 // 2AM
				tzi.DaylightDate.wMinute = 0;
				tzi.DaylightDate.wMilliseconds = 0;

				tzi.StandardDate.wYear = 0;		 // always 0
                tzi.StandardDate.wDayOfWeek = (ushort)this.wdComboBox.SelectedIndex;
                tzi.StandardDate.wDay = (ushort)this.posComboBox.SelectedIndex;
                tzi.StandardDate.wMonth = (ushort)(this.monthComboBox.SelectedIndex + 1);
				tzi.StandardDate.wHour = 2;		 // 2AM
				tzi.StandardDate.wMinute = 0;
				tzi.StandardDate.wMilliseconds = 0;

                // -60 is moving back 1 hour
                tzi.DaylightBias = (int)this.tzDSTBiasNumericUpDown.Value;
                // (usually 0, as non-DST has no "extra" bias)
                tzi.StandardBias = (int)this.tzbiasNumericUpDown.Value;

                //TODO: update the bias of the timezone:
                // tzi.Bias = -120;	// GMT -2:00

				uint setTZResult = SetTimeZoneInformation(ref tzi);
				if (setTZResult != 0)
				{
					this.logTextBox.Text += "\r\nSuccess!\r\n" + TZIToString(tzi);
					// Write permanently
					Registry.LocalMachine.Flush();
				}
				else
				{
					this.logTextBox.Text += "\r\nError setting timezone information.";
					int lastError = Marshal.GetLastWin32Error();
					this.logTextBox.Text += "\r\nErrorCode: " + lastError;

					try
					{
						StringBuilder sb = new StringBuilder(500);
						FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, null, (uint)lastError, 0, sb, 500, null);
						this.logTextBox.Text += "\r\nErrorMsg: " + sb.ToString();
					}
					catch
					{
						this.logTextBox.Text = "\r\nCould not format error message.";
					}
				}
			}
		}

        private bool firstTimeInitCalled = false;

        private void Form1_Activated(object sender, EventArgs e)
        {
            if (!this.firstTimeInitCalled) {
                /* Load the timezone information */
                this.tzi = new TIME_ZONE_INFORMATION();
                Form1.GetTimeZoneInformation(out tzi);
                this.logTextBox.Text = this.TZIToString(tzi);
                this.logTextBox.Refresh();

                /* Display it in the GUI */
                this.displayCurrentTZ();
                this.firstTimeInitCalled = true;
            }
        }

        private void displayCurrentTZ()
        {
            /* Show the timezone in the GUI */
            this.tzNameTextBox.Text = tzi.StandardName;
            this.tzbiasNumericUpDown.Value = this.tzi.StandardBias;

            this.posDSTComboBox.SelectedIndex = tzi.DaylightDate.wDay;
            this.wdDSTComboBox.SelectedIndex = tzi.DaylightDate.wDayOfWeek;
            this.monthDSTComboBox.SelectedIndex = tzi.DaylightDate.wMonth-1;
            this.timeDSTLabel.Text = String.Format("{0}:{0}", 
                tzi.DaylightDate.wHour, tzi.DaylightDate.wMinute);

            this.dstNameTextBox.Text = tzi.DaylightName;

            this.posComboBox.SelectedIndex = tzi.StandardDate.wDay;
            this.wdComboBox.SelectedIndex = tzi.StandardDate.wDayOfWeek;
            this.monthComboBox.SelectedIndex = tzi.StandardDate.wMonth-1;
            this.timeDSTEndLabel.Text = String.Format("{0}:{0}", 
                tzi.StandardDate.wHour, tzi.StandardDate.wMinute);

            this.tzDSTBiasNumericUpDown.Value = this.tzi.DaylightBias;

            // ??? whta is this?
            this.tzBias.Text = this.tzi.Bias.ToString();
        }

        private void aboutMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Created by Shalom Craimer\r\n(http://shalom.craimer.org)\r\nDo whatever you want with the source code, but I would appreciate getting a little credit. Thanks, and enjoy!", "About");
        }
	}
}
